resource "openstack_lb_loadbalancer_v2" "loadbalancer_app" {
  name           = "${var.system_name}-loadbalancer-app"
  vip_network_id = openstack_networking_network_v2.network_internal_app.id
}

# HTTP Setup

resource "openstack_lb_listener_v2" "listener_http_app" {
  name            = "${var.system_name}-app_listener_http"
  protocol        = "TCP"
  protocol_port   = 80
  loadbalancer_id = openstack_lb_loadbalancer_v2.loadbalancer_app.id
  default_pool_id = openstack_lb_pool_v2.pool_http_app.id
}

resource "openstack_lb_pool_v2" "pool_http_app" {
  name            = "${var.system_name}-pool_http_app"
  protocol        = "TCP"
  lb_method       = "SOURCE_IP_PORT"
  loadbalancer_id = openstack_lb_loadbalancer_v2.loadbalancer_app.id
}

resource "openstack_lb_member_v2" "member_http_app" {
  name          = "${var.system_name}-member_http_app-${count.index}"
  count         = length(openstack_compute_instance_v2.server_app)
  pool_id       = openstack_lb_pool_v2.pool_http_app.id
  address       = openstack_compute_instance_v2.server_app.*.access_ip_v4[count.index]
  protocol_port = 80
}

resource "openstack_lb_monitor_v2" "monitor_http_app" {
  name        = "${var.system_name}-monitor_http_app"
  pool_id     = openstack_lb_pool_v2.pool_http_app.id
  type        = "TCP"
  delay       = 10
  timeout     = 5
  max_retries = 3
}

# HTTPS Setup

resource "openstack_lb_listener_v2" "listener_https_app" {
  name            = "${var.system_name}-listener_https_app"
  protocol        = "TCP"
  protocol_port   = 443
  loadbalancer_id = openstack_lb_loadbalancer_v2.loadbalancer_app.id
  default_pool_id = openstack_lb_pool_v2.pool_https_app.id
}

resource "openstack_lb_pool_v2" "pool_https_app" {
  name            = "${var.system_name}-pool_https_app"
  protocol        = "TCP"
  lb_method       = "SOURCE_IP_PORT"
  loadbalancer_id = openstack_lb_loadbalancer_v2.loadbalancer_app.id
}

resource "openstack_lb_member_v2" "member_https_app" {
  name          = "${var.system_name}-member_https_app-${count.index}"
  count         = length(openstack_compute_instance_v2.server_app)
  pool_id       = openstack_lb_pool_v2.pool_https_app.id
  address       = openstack_compute_instance_v2.server_app.*.access_ip_v4[count.index]
  protocol_port = 443
}

resource "openstack_lb_monitor_v2" "monitor_https_app" {
  name        = "${var.system_name}-monitor_https_app"
  pool_id     = openstack_lb_pool_v2.pool_https_app.id
  type        = "TCP"
  delay       = 10
  timeout     = 5
  max_retries = 3
}

# SSH ACCESS

resource "openstack_lb_listener_v2" "listener_ssh_app" {
  count           = length(openstack_compute_instance_v2.server_app)
  name            = "${var.system_name}-listener_ssh_app-${count.index}"
  protocol        = "TCP"
  protocol_port   = 22000 + count.index
  loadbalancer_id = openstack_lb_loadbalancer_v2.loadbalancer_app.id
  default_pool_id = openstack_lb_pool_v2.pool_ssh_app.*.id[count.index]
}

resource "openstack_lb_pool_v2" "pool_ssh_app" {
  count           = length(openstack_compute_instance_v2.server_app)
  name            = "${var.system_name}-pool_ssh_app-${count.index}"
  protocol        = "TCP"
  lb_method       = "SOURCE_IP_PORT"
  loadbalancer_id = openstack_lb_loadbalancer_v2.loadbalancer_app.id
}

resource "openstack_lb_member_v2" "member_ssh_app" {
  count         = length(openstack_compute_instance_v2.server_app)
  name          = "${var.system_name}-member_ssh_app-${count.index}"
  pool_id       = openstack_lb_pool_v2.pool_ssh_app.*.id[count.index]
  address       = openstack_compute_instance_v2.server_app.*.access_ip_v4[count.index]
  protocol_port = 22
}

resource "openstack_lb_monitor_v2" "monitor_ssh_app" {
  count       = length(openstack_compute_instance_v2.server_app)
  name        = "${var.system_name}-monitor_ssh_app-${count.index}"
  pool_id     = openstack_lb_pool_v2.pool_ssh_app.*.id[count.index]
  type        = "TCP"
  delay       = 10
  timeout     = 5
  max_retries = 3
}
